module Game.CInstructionsController;
import derelict.opengl3.gl3;
import Tractor.Tractor;
import Game.CGameInfo, Game.IUIEvents;
import Render.CScene;
import Text.CProcessor, Text.CText;

class CInstructionsController
{
	private ITractor mTractor;
	private IUIEvents mUIEvents;
	private CGameInfo mGameInfo;
	public bool mInitialised = false;
	private ITexture mTitleImage;
	private IFont mFont;
	private CScene mScene;
	private uint mTitleSprite;
	private int mScrollPos = 0;
	private int mTextYSize;

	public this(ITractor aTractor, IUIEvents aUIEvents, CGameInfo aGameInfo)
	{
		mTractor = aTractor;
		mUIEvents = aUIEvents;
		mGameInfo = aGameInfo;
	}



	public ~this()
	{
		delete mScene;
	}



	private void Initialise()
	{
		mInitialised = true;
		mTitleImage = mTractor.Textures.LoadImage("Title.png");
		glBindTexture(GL_TEXTURE_2D, mTitleImage.ID);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		mFont = mTractor.Fonts.LoadFont(mGameInfo.MenuFont);
		mScene = new CScene(mTractor, 200);
		mTitleSprite = mScene.CreateSprite();
		mScene.SetSprite(mTitleSprite, 0, 0, 960, 576, mTitleImage);
		auto Processor = new CProcessor();
		enum int XMargin = 140;
		auto Text = new CText(mTractor, Processor, mScene, 576, XMargin, 30,
				960 - XMargin * 2, 0, 2, 0);
		Text.SetFont(mFont, mGameInfo.MenuFontSize);
		auto File = mTractor.Core.OpenDataFile("Instructions.txt");
		while (!File.eof)
		{
			auto Line = File.readLine();
			if (Line == "-")
			{
				Text.SetFont(mFont, 8);
				Text.Write(" \n");
				Text.SetFont(mFont, mGameInfo.MenuFontSize);
			}
			else if (Line == "--")
			{
				Text.SetFont(mFont, mGameInfo.MenuFontSize + 10);
				Text.Write(" \n");
				Text.SetFont(mFont, mGameInfo.MenuFontSize);
			}
			else
				Text.Write(Line.idup ~ '\n');
		}
		mTractor.Core.CloseFile(File);
		Text.End(mTextYSize);
	}



	public void Render()
	{
		if (!mInitialised)
			Initialise();
		mScene.Render();
	}



	public void OnKeyDown(EKey aKey)
	{
		switch (aKey)
		{
			case EKey.Escape:
			case EKey.Enter:
				mUIEvents.MainMenu();
				break;
			case EKey.Up:
				Scroll(-200);
				break;
			case EKey.Down:
				Scroll(200);
				break;
			default:
				break;
		}
	}



	private void Scroll(int aDelta)
	{
		if (aDelta < 0)
		{
			if (mScrollPos < 0)
				return;
			if (mScrollPos + aDelta < 0)
				aDelta = -mScrollPos;
		}
		else
		{
			if (mScrollPos >= mTextYSize - (576 - 60))
				return;
			if (mScrollPos + aDelta > mTextYSize - (576 - 60))
				aDelta = mTextYSize - (576 - 60) - mScrollPos;
		}
		mScrollPos += aDelta;
		for (uint i = 1; i < mScene.NumSprites; i++)
			mScene.MoveSprite(i, 0, -aDelta);
		mTractor.Core.Redraw();
	}
}