﻿module Render.CDisplay;
import std.typecons;
import derelict.opengl3.gl3;
import Tractor.Tractor;
import QBasic.IGraphicsExternal;
import Render.CScene;

class CDisplay : IGraphicsExternal
{
	private ITractor mTractor;
	private CScene mScene;
	private uint mDisplaySprite;
	private ITexture mTexture;
	private Rebindable!(const(ubyte[])) mScreenData;
	private int mScreenXSize;
	private int mScreenYSize;
	private uint[] mTextureData;
	private uint[256] mPalette;

	public this(ITractor aTractor)
	{
		mTractor = aTractor;
	}



	public void Initialise()
	{
		mScene = new CScene(mTractor, 1);
		mDisplaySprite = mScene.CreateSprite();
	}



	public void Reset()
	{
	}



	public void SetScreen(const(ubyte[]) aScreenData, int aScreenXSize, int aScreenYSize)
	{
		mScreenData = aScreenData;
		mScreenXSize = aScreenXSize;
		mScreenYSize = aScreenYSize;
		mTextureData = new uint[mScreenXSize * mScreenYSize];
		mTexture = mTractor.Textures.CreateTexture(mTextureData, mScreenXSize, mScreenYSize);
		glBindTexture(GL_TEXTURE_2D, mTexture.ID);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		mScene.SetSprite(mDisplaySprite, 0, 0, 960, 576, mTexture);
	}



	public void Render()
	{
		GenerateTextureData();
		mScene.Render();
	}



	private void GenerateTextureData()
	{
		for (int i = 0; i < mTextureData.length; i++)
			mTextureData[i] = mPalette[mScreenData[i]];
		mTexture.SetData(mTextureData);
	}



	public void SetColour(int aIndex, int aR, int aG, int aB)
	{
		mPalette[aIndex] = (aR * 4 + aR / 16) | ((aG * 4 + aG / 16) << 8) |
				((aB * 4 + aB / 16) << 16) | 0xff000000;
	}
}