﻿module Text.CText;
import std.utf;
import derelict.opengl3.gl3;
import freetype.ft;
import Tractor.Tractor;
import Render.CScene;
import Text.CProcessor, Text.CTextFragment, Text.CWrite;

class CText
{
	private ITractor mTractor;
	private CProcessor mProcessor;
	private CScene mScene;
	private int mWindowYSize;
	private int mOriginX;
	private int mOriginY;
	private int mXSize;
	private float mXAlign;
	private int mLineSpacing;
	private float mYAlign;
	private IFont mFont = null;
	private int mFontSize = 0;
	private CColour mColour = new CColour(1, 1, 1);
	private float mAlpha = 1;
	private CWrite[] mWrites;

	public this(ITractor aTractor, CProcessor aProcessor, CScene aScene, int aWindowYSize,
			int aOriginX, int aOriginY, int aXSize, float aXAlign, int aLineSpacing,
			float aYAlign)
	{
		mTractor = aTractor;
		mProcessor = aProcessor;
		mScene = aScene;
		mWindowYSize = aWindowYSize;
		mOriginX = aOriginX;
		mOriginY = aOriginY;
		mXSize = aXSize;
		mXAlign = aXAlign;
		mLineSpacing = aLineSpacing;
		mYAlign = aYAlign;
	}



	public void SetFont(IFont aFont, int aSize = 0)
	{
		mFont = aFont;
		if (aSize != 0)
			mFontSize = aSize;
	}



	public void SetColour(CColour aColour)
	{
		mColour = aColour;
	}



	public void SetAlpha(float aAlpha)
	{
		mAlpha = aAlpha;
	}



	public void Write(string aText)
	in
	{
		assert(aText != "");
	}
	body
	{
		mWrites ~= new CWrite(mFont, ToFTSize(mFontSize), mColour, mAlpha, aText);
	}



	public void End()
	{
		int YSize;
		End(YSize);
	}



	public void End(out int aYSize)
	{
		auto WrapXSize = mXSize == -1 ? -1 : ToFTSize(mXSize);
		CTextFragment[] Fragments = mProcessor.Process(ToFTX(mOriginX), ToFTY(mOriginY),
				WrapXSize, mXAlign, ToFTSize(mLineSpacing), mYAlign, mWrites);
		int MinY = int.max, MaxY = int.min;
		foreach (CTextFragment Fragment; Fragments)
		{
			if (Fragment.IsImage)
			{
				int YSize = ToWindowSize(Fragment.YSize);
				mScene.SetSprite(mScene.CreateSprite(), ToWindowX(Fragment.X),
						ToWindowY(Fragment.Y) - YSize, ToWindowSize(Fragment.XSize),
						YSize, Fragment.Texture);
			}
			else
			{
				FT_Vector Pen;
				Pen.x = Fragment.X;
				Pen.y = Fragment.BaselineY;
				size_t Index = 0;
				int TextureX;
				int TextureY;
				int TextureXSize;
				int TextureYSize;
				PixelBoundBox(Fragment.X, Fragment.Y, Fragment.XSize, Fragment.YSize,
						TextureX, TextureY, TextureXSize, TextureYSize);
				uint[] TextureData = new uint [TextureXSize * TextureYSize];
				FT_Error FTResult = FT_Set_Char_Size(Fragment.Font.Face, 0, Fragment.FontSize, 0, 0);
				assert(FTResult == 0);
				while (Index < Fragment.Text.length)
				{
					dchar Char = decode(Fragment.Text, Index);
					FT_Set_Transform(Fragment.Font.Face, null, &Pen);
					FTResult = FT_Load_Char(Fragment.Font.Face, cast(FT_ULong)Char,
							FT_LOAD_RENDER);
					assert(FTResult == 0);
					Pen.x += Fragment.Font.Face.glyph.advance.x;
					Pen.y += Fragment.Font.Face.glyph.advance.y;
					FT_Glyph Glyph;
					FTResult = FT_Get_Glyph(Fragment.Font.Face.glyph, &Glyph);
					assert(FTResult == 0);
					assert(Glyph.format == FT_Glyph_Format.FT_GLYPH_FORMAT_BITMAP);
					FT_BitmapGlyph BitmapGlyph = cast(FT_BitmapGlyph)Glyph;
					int OffsetX = Fragment.Font.Face.glyph.bitmap_left - TextureX;
					int OffsetY = mWindowYSize - Fragment.Font.Face.glyph.bitmap_top - TextureY;
					for (int y = 0; y < BitmapGlyph.bitmap.rows; y++)
					{
						for (int x = 0; x < BitmapGlyph.bitmap.width; x++)
						{
							ubyte Intensity = BitmapGlyph.bitmap.buffer[x + y * BitmapGlyph.bitmap.pitch];
							float IntensityMultiplier = cast(float)Intensity / 255;
							auto Colour = new CColour(Fragment.Colour.R * IntensityMultiplier,
									Fragment.Colour.G * IntensityMultiplier,
									Fragment.Colour.B * IntensityMultiplier);
							if (OffsetX + x >= 0 && OffsetX + x < TextureXSize && OffsetY + y >= 0 && OffsetY + y < TextureYSize)
								TextureData[(OffsetX + x) + (OffsetY + y) * TextureXSize] = Intensity << 24 | Colour.Int();
						}
					}
					FT_Done_Glyph(Glyph);
				}
				auto Texture = mTractor.Textures.CreateTexture(TextureData, TextureXSize, TextureYSize);
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
				mScene.SetSprite(mScene.CreateSprite(), TextureX, TextureY, TextureXSize,
						TextureYSize, Texture);
				if (TextureY < MinY)
					MinY = TextureY;
				if (TextureY + TextureYSize > MaxY)
					MaxY = TextureY + TextureYSize;
			}
		}
		aYSize = MaxY - MinY;
		mWrites = [];
	}



	private void PixelBoundBox(FT_Pos aX0, FT_Pos aY0, FT_F26Dot6 aXSize, FT_F26Dot6 aYSize,
		out int aPixelX0, out int aPixelY0, out int aPixelXSize, out int aPixelYSize)
	{
		aPixelX0 = aX0 / 64;
		aPixelY0 = (mWindowYSize * 64 - (aY0 + aYSize)) / 64;
		int X1 = aX0 + aXSize;
		aPixelXSize = X1 % 64 == 0 ? X1 / 64 - aPixelX0 : X1 / 64 + 1 - aPixelX0;
		int Y1 = mWindowYSize * 64 - aY0;
		aPixelYSize = Y1 % 64 == 0 ? Y1 / 64 - aPixelY0 : Y1 / 64 + 1 - aPixelY0;
	}



	private FT_Pos ToFTX(int aX)
	{
		return aX * 64;
	}



	private FT_Pos ToFTY(int aY)
	{
		return (mWindowYSize - aY) * 64;
	}



	private FT_F26Dot6 ToFTSize(int aSize)
	{
		return aSize * 64;
	}



	private int ToWindowX(FT_Pos aX)
	{
		return aX / 64;
	}



	private int ToWindowY(FT_Pos aY)
	{
		return mWindowYSize - aY / 64;
	}



	private int ToWindowSize(FT_F26Dot6 aSize)
	{
		return aSize / 64;
	}
}