﻿module QBasic.Interpreter.IInterpreterState;
import QBasic.Interpreter.CEvaluator, QBasic.Interpreter.CReference,
		QBasic.Program.CIdentifier, QBasic.Program.CName, QBasic.Program.CType,
		QBasic.Program.CValue, QBasic.Program.EExtensionCommand,
		QBasic.Program.Expressions.CExpression, QBasic.Program.Instructions.IInstruction,
		QBasic.System.IFileSystem, QBasic.System.IGraphics;

interface IInterpreterState
{
	CEvaluator GetEvaluator();

	IInstruction GetInstruction();
	int GetInstructionStage();

	void NextInstruction(int aNextInstruction);

	bool Expression(CExpression aExpression, out bool aBlocking);
	CValue GetExpressionResult();

	void CallProcedure(CIdentifier aName, CValue[] aArguments);
	CValue GetFunctionResult();
	void ExtensionCommand(EExtensionCommand aCommand, CValue[] aArguments);

	CReference TakeArrayReference(CName aName, CType aType);
	CReference TakeReference(CName aName, immutable(int[]) aSubscripts, int aRecordOffset,
			CType aType);
	CValue ReadTarget(CReference aReference);
	CValue ReadVariable(CReference aReference);
	void WriteVariable(CReference aTarget, CValue aValue);
	void ResizeArray(CName aName, const(int[]) aNewDimensions);

	void SetStack(int aOffset, CValue aValue);
	CValue GetStack(int aOffset);

	IFileSystem FileSystem();
	IGraphics Graphics();

	void SetError(string aMessage);

	void SetDefSeg(CReference aReference);
	bool WriteData(const(ubyte[]) aData);
	bool WriteData(CReference aReference, const(ubyte[]) aData);
	immutable(ubyte[]) ReadData(CReference aReference, int aByteSize);
	int ValueSize(CReference aReference);
}