﻿module QBasic.Interpreter.Instructions.CFileGetInterpreter;
import QBasic.Interpreter.CReference, QBasic.Interpreter.IInterpreterState,
		QBasic.Interpreter.Instructions.IInstructionInterpreter, QBasic.Program.CType,
		QBasic.Program.CValue, QBasic.Program.Instructions.CFileGetInstruction;

class CFileGetInterpreter : IInstructionInterpreter
{
	private int mFileNumber;

	public bool Execute(IInterpreterState aState, out bool aBlocking)
	{
		auto Instruction = cast(CFileGetInstruction)aState.GetInstruction();
		for (int Stage = aState.GetInstructionStage(); ; Stage++)
		{
			switch (Stage)
			{
				case 0:
					if (!aState.Expression(Instruction.FileNumberExpression, aBlocking))
						return false;
					break;
				case 1:
					mFileNumber = aState.GetExpressionResult().IntegralValue;
					if (!aState.Expression(Instruction.TargetExpression, aBlocking))
						return false;
					break;
				case 2:
					auto Target = aState.GetExpressionResult().Reference;
					int Size = aState.ValueSize(Target);
					ubyte[] Data;
					if (!aState.FileSystem.Get(mFileNumber, Size, Data))
						return false;
					aState.WriteData(Target, Data);
					return true;
				default:
					assert(false);
			}
		}
	}
}