﻿module QBasic.Interpreter.Instructions.CFilePutInterpreter;
import QBasic.Interpreter.CReference, QBasic.Interpreter.IInterpreterState,
		QBasic.Interpreter.Instructions.IInstructionInterpreter, QBasic.Program.CType,
		QBasic.Program.CValue, QBasic.Program.Instructions.CFilePutInstruction;

class CFilePutInterpreter : IInstructionInterpreter
{
	private int mFileNumber;

	public bool Execute(IInterpreterState aState, out bool aBlocking)
	{
		auto Instruction = cast(CFilePutInstruction)aState.GetInstruction();
		for (int Stage = aState.GetInstructionStage(); ; Stage++)
		{
			switch (Stage)
			{
				case 0:
					if (!aState.Expression(Instruction.FileNumberExpression, aBlocking))
						return false;
					break;
				case 1:
					mFileNumber = aState.GetExpressionResult().IntegralValue;
					if (!aState.Expression(Instruction.SourceExpression, aBlocking))
						return false;
					break;
				case 2:
					auto Source = aState.GetExpressionResult().Reference;
					auto Data = aState.ReadData(Source, aState.ValueSize(Source));
					if (Data is null)
						return false;
					return aState.FileSystem.Put(mFileNumber, Data);
				default:
					assert(false);
			}
		}
	}
}