﻿module QBasic.Interpreter.Instructions.CForInitInterpreter;
import QBasic.Interpreter.IInterpreterState, QBasic.Interpreter.Instructions.IInstructionInterpreter,
		QBasic.Interpreter.CReference, QBasic.Program.CType, QBasic.Program.CValue,
		QBasic.Program.Instructions.CForInitInstruction;

class CForInitInterpreter : IInstructionInterpreter
{
	private CReference mTarget;

	public bool Execute(IInterpreterState aState, out bool aBlocking)
	{
		auto Instruction = cast(CForInitInstruction)aState.GetInstruction();
		for (int Stage = aState.GetInstructionStage(); ; Stage++)
		{
			switch (Stage)
			{
				case 0:
					if (!aState.Expression(Instruction.TargetExpression, aBlocking))
						return false;
					break;
				case 1:
					mTarget = aState.GetExpressionResult().Reference;
					aState.SetStack(Instruction.GetStackOffset(), aState.GetExpressionResult());
					if (!aState.Expression(Instruction.GetStartExpression(), aBlocking))
						return false;
					break;
				case 2:
					aState.WriteVariable(mTarget, aState.GetExpressionResult());
					if (!aState.Expression(Instruction.GetEndExpression(), aBlocking))
						return false;
					break;
				case 3:
					aState.SetStack(Instruction.GetStackOffset() + 1, aState.GetExpressionResult());
					if (Instruction.GetStepExpression() is null)
					{
						aState.SetStack(Instruction.GetStackOffset() + 2, CValue.Integer(1));
						return true;
					}
					if (!aState.Expression(Instruction.GetStepExpression(), aBlocking))
						return false;
					break;
				case 4:
					aState.SetStack(Instruction.GetStackOffset() + 2,
							aState.GetExpressionResult());
					return true;
				default:
					throw new Throwable("Invalid instruction stage");
			}
		}
	}
}