﻿module QBasic.Interpreter.Instructions.CMidInterpreter;
import QBasic.Interpreter.IInterpreterState, QBasic.Interpreter.CReference,
		QBasic.Interpreter.Instructions.IInstructionInterpreter, QBasic.Program.CValue,
		QBasic.Program.Instructions.CMidInstruction;

class CMidInterpreter : IInstructionInterpreter
{
	private CReference mTarget;
	private int mStart;

	public bool Execute(IInterpreterState aState, out bool aBlocking)
	{
		auto Instruction = cast(CMidInstruction)aState.GetInstruction();
		for (int Stage = aState.GetInstructionStage(); ; Stage++)
		{
			switch (Stage)
			{
				case 0:
					if (!aState.Expression(Instruction.TargetExpression, aBlocking))
						return false;
					break;
				case 1:
					mTarget = aState.GetExpressionResult().Reference;
					if (!aState.Expression(Instruction.StartExpression, aBlocking))
						return false;
					break;
				case 2:
					mStart = aState.GetExpressionResult().IntegralValue - 1;
					if (!aState.Expression(Instruction.Expression, aBlocking))
						return false;
					break;
				case 3:
					auto CurrentValue = aState.ReadVariable(mTarget).StringValue;
					auto String = aState.GetExpressionResult().StringValue;
					if (mStart < 0 || mStart + String.length > CurrentValue.length)
					{
						aState.SetError("Invalid substring range.");
						return false;
					}
					ubyte[] NewString = CurrentValue.dup;
					NewString[mStart .. mStart + String.length] = String[];
					aState.WriteVariable(mTarget, CValue.String(NewString.idup));
					return true;
				default:
					assert(false);
			}
		}
	}
}