﻿module QBasic.Interpreter.Instructions.COutInterpreter;
import QBasic.Interpreter.IInterpreterState, QBasic.Interpreter.Instructions.IInstructionInterpreter,
		QBasic.Program.Instructions.COutInstruction;

class COutInterpreter : IInstructionInterpreter
{
	private int mPort;

	public bool Execute(IInterpreterState aState, out bool aBlocking)
	{
		auto Instruction = cast(COutInstruction)aState.GetInstruction();
		for (int Stage = aState.GetInstructionStage(); ; Stage++)
		{
			switch (Stage)
			{
				case 0:
					if (!aState.Expression(Instruction.PortExpression, aBlocking))
						return false;
					break;
				case 1:
					mPort = aState.GetExpressionResult().IntegralValue;
					if (!aState.Expression(Instruction.DataExpression, aBlocking))
						return false;
					break;
				case 2:
					int Data = aState.GetExpressionResult().IntegralValue;
					if (Data < 0 || Data > 255)
					{
						aState.SetError("Invalid OUT data value.");
						return false;
					}
					switch (mPort)
					{
						case 0x3c8:
							aState.Graphics.SetCurrentColourIndex(Data);
							break;
						case 0x3c9:
							if (!aState.Graphics.SetCurrentColourElement(Data))
							{
								aState.SetError("Invalid colour element value.");
								return false;
							}
							break;
						default:
							aState.SetError("OUT port not implemented.");
							return false;
					}
					return true;
				default:
					assert(false);
			}
		}
	}
}