﻿module QBasic.Interpreter.Instructions.CSelectCaseInterpreter;
import QBasic.Interpreter.IInterpreterState, QBasic.Interpreter.Instructions.IInstructionInterpreter,
	QBasic.Program.CValue, QBasic.Program.CType, QBasic.Program.EOperator,
		QBasic.Program.Instructions.CSelectCase, QBasic.Program.Instructions.CSelectCaseInstruction;

class CSelectCaseInterpreter : IInstructionInterpreter
{
	private int mCase = 0;
	private bool mEnd = false;

	public bool Execute(IInterpreterState aState, out bool aBlocking)
	{
		auto Instruction = cast(CSelectCaseInstruction)aState.GetInstruction();
		for (int Stage = aState.GetInstructionStage(); ; Stage++)
		{
			if (Stage != 0)
			{
				CValue Value = aState.GetExpressionResult();
				CSelectCase Case = Instruction.GetCases()[mCase];
				if (Case.GetType() == ECaseType.Range && !mEnd)
				{
					CValue Result = aState.GetEvaluator().BinaryOperator(aState.GetStack(
							Instruction.StackOffset), Value, EOperator.CompareGreaterOrEqual,
							EPrimitiveType.Integer);
					if (Result is null)
						return false;
					if (Result.IntegralValue != 0)
						mEnd = true;
					else
					{
						mCase++;
						if (mCase == Instruction.GetCases().length)
						{
							aState.NextInstruction(Instruction.GetNextInstruction());
							return true;
						}
					}
				}
				else
				{
					final switch (Case.GetType())
					{
						case ECaseType.Normal:
							CValue Result = aState.GetEvaluator().BinaryOperator(aState.
									GetStack(Instruction.StackOffset), Value,
									EOperator.CompareEqual, EPrimitiveType.Integer);
							if (Result is null)
								return false;
							if (Result.IntegralValue != 0)
								return true;
							break;
						case ECaseType.Range:
							CValue Result = aState.GetEvaluator().BinaryOperator(aState.
									GetStack(Instruction.StackOffset), Value,
									EOperator.CompareLessOrEqual, EPrimitiveType.Integer);
							if (Result is null)
								return false;
							if (Result.IntegralValue != 0)
								return true;
							break;
						case ECaseType.Is:
							EOperator Operator;
							final switch (Case.GetOperator())
							{
								case ECaseIsOperator.Less:
									Operator = EOperator.CompareLess;
									break;
								case ECaseIsOperator.LessOrEqual:
									Operator = EOperator.CompareLessOrEqual;
									break;
								case ECaseIsOperator.Greater:
									Operator = EOperator.CompareGreater;
									break;
								case ECaseIsOperator.GreaterOrEqual:
									Operator = EOperator.CompareGreaterOrEqual;
									break;
								case ECaseIsOperator.Equal:
									Operator = EOperator.CompareEqual;
									break;
								case ECaseIsOperator.NotEqual:
									Operator = EOperator.CompareNotEqual;
									break;
							}
							CValue Result = aState.GetEvaluator().BinaryOperator(aState.
									GetStack(Instruction.StackOffset), Value,
									EOperator.CompareLessOrEqual, EPrimitiveType.Integer);
							if (Result is null)
								return false;
							if (Result.IntegralValue != 0)
								return true;
							break;
					}
					mCase++;
					if (mCase == Instruction.GetCases().length)
					{
						aState.NextInstruction(Instruction.GetNextInstruction());
						return true;
					}
					mEnd = false;
				}
			}
			CSelectCase Case = Instruction.GetCases()[mCase];
			if (Case.GetType() == ECaseType.Range && mEnd)
			{
				if (!aState.Expression(Case.GetEndExpression(), aBlocking))
					return false;
			}
			else
			{
				if (!aState.Expression(Case.GetExpression(), aBlocking))
					return false;
			}
		}
	}
}