﻿module QBasic.Parser.Expressions.CExpressionNode;
import QBasic.Parser.Expressions.Nodes, QBasic.Program.CType;

enum EExpressionNodeType { BinaryOperator, BuiltinFunction, UnaryOperator, UserFunction,
		Value, Variable };

class CExpressionNode
{
	private bool mComplete = false;
	private int mSubtreeSize;

	public abstract EExpressionNodeType NodeType();
	public abstract EPrimitiveType Type();



	public bool Complete()
	{
		return mComplete;
	}



	public void SetComplete(int aSubtreeSize)
	in
	{
		assert(!mComplete);
	}
	body
	{
		mComplete = true;
		mSubtreeSize = aSubtreeSize;
	}



	public int SubtreeSize()
	in
	{
		assert(mComplete);
	}
	body
	{
		return mSubtreeSize;
	}



	public CBinaryOperatorExpressionNode AsBinaryOperator()
	in
	{
		assert(NodeType == EExpressionNodeType.BinaryOperator);
	}
	body
	{
		return cast(CBinaryOperatorExpressionNode)this;
	}



	public CBuiltinFunctionExpressionNode AsBuiltinFunction()
	in
	{
		assert(NodeType == EExpressionNodeType.BuiltinFunction);
	}
	body
	{
		return cast(CBuiltinFunctionExpressionNode)this;
	}



	public CUnaryOperatorExpressionNode AsUnaryOperator()
	in
	{
		assert(NodeType == EExpressionNodeType.UnaryOperator);
	}
	body
	{
		return cast(CUnaryOperatorExpressionNode)this;
	}



	public CUserFunctionExpressionNode AsUserFunction()
	in
	{
		assert(NodeType == EExpressionNodeType.UserFunction);
	}
	body
	{
		return cast(CUserFunctionExpressionNode)this;
	}



	public CValueExpressionNode AsValue()
	in
	{
		assert(NodeType == EExpressionNodeType.Value);
	}
	body
	{
		return cast(CValueExpressionNode)this;
	}



	public CVariableExpressionNode AsVariable()
	in
	{
		assert(NodeType == EExpressionNodeType.Variable);
	}
	body
	{
		return cast(CVariableExpressionNode)this;
	}
}