﻿module QBasic.Parser.Expressions.CVariableExpressionNode;
import std.typecons;
import QBasic.Parser.Expressions.CExpressionNode, QBasic.Program.CName, QBasic.Program.CType,
		QBasic.Program.Expressions.CExpression;

class CVariableExpressionNode : CExpressionNode
{
	private CName mName;
	private CType mType;
	private CExpression[] mSubscripts;
	private int mRecordOffset = 0;
	private CType mRecordFieldType = null;
	private bool mArrayReference = false;
	private bool mReference = false;

	invariant()
	{
		assert(!mArrayReference || !mReference);
	}

	public this(CName aName, CType aType)
	in
	{
		assert(aName !is null && aType !is null);
	}
	body
	{
		mName = aName;
		mType = aType;
	}



	public override EExpressionNodeType NodeType()
	{
		return EExpressionNodeType.Variable;
	}



	public override EPrimitiveType Type()
	{
		auto Type = TargetType;
		return Type.Type == EType.Record ? EPrimitiveType.Reference : Type.PrimitiveType;
	}



	public CName Name()
	{
		return mName;
	}



	public CType VariableType()
	{
		return mType;
	}



	public CType TargetType()
	out(rResult)
	{
		assert(rResult !is null);
	}
	body
	{
		auto Type = mType;
		if (Type.Type == EType.Array || Type.Type == EType.ArrayReference)
			Type = Type.SubType;
		if (Type.Type == EType.Record && mRecordFieldType !is null)
			Type = mRecordFieldType;
		return Type;
	}



	public CExpression[] Subscripts()
	{
		return mSubscripts;
	}



	public void AddSubscript(CExpression aSubscript)
	{
		mSubscripts ~= [ aSubscript ];
	}



	public int RecordOffset()
	{
		return mRecordOffset;
	}



	public void SetRecordOffset(int aOffset, CType aFieldType)
	in
	{
		assert(aOffset >= 0 && aFieldType !is null);
	}
	body
	{
		mRecordOffset = aOffset;
		mRecordFieldType = aFieldType;
	}



	public bool ArrayReference()
	{
		return mArrayReference;
	}



	/**
	Instructs the interpreter to take a reference to the array represented by this term.
	*/
	public void SetArrayReference()
	{
		mArrayReference = true;
	}



	public bool Reference()
	{
		return mReference;
	}



	/**
	Instructs the interpreter to take a reference to the variable represented by this term,
	instead of reading the variable.
	*/
	public void SetReference()
	{
		mReference = true;
	}
}