﻿module QBasic.Program.CIdentifier;
import std.string;

enum ESuffix { None, Integer, Long, Single, String };

class CIdentifier
{
	public immutable string Name;
	public immutable ESuffix Suffix;

	invariant()
	{
		assert(Name != "");
	}



	public this(string aIdentifier)
	{
		ESuffix Suffix;
		switch (aIdentifier[$ - 1])
		{
			case '%':
				Suffix = ESuffix.Integer;
				break;
			case '&':
				Suffix = ESuffix.Long;
				break;
			case '!':
				Suffix = ESuffix.Single;
				break;
			case '$':
				Suffix = ESuffix.String;
				break;
			default:
				Suffix = ESuffix.None;
				break;
		}
		this.Suffix = Suffix;
		this.Name = Suffix == ESuffix.None ? aIdentifier : aIdentifier[0 .. $ - 1];
	}



	public string Identifier() const
	{
		final switch (Suffix)
		{
			case ESuffix.Integer:
				return Name ~ "%";
			case ESuffix.Long:
				return Name ~ "&";
			case ESuffix.Single:
				return Name ~ "!";
			case ESuffix.String:
				return Name ~ "$";
			case ESuffix.None:
				return Name;
		}
	}



	public override hash_t toHash() const
	{
		return Name.length;   // TODO: make a better hash function
	}



	public override bool opEquals(Object o) const
	{
		if (o is null) return false;
		assert(typeid(this) == typeid(o));
		auto Identifier = cast(CIdentifier)o;
		return icmp(Name, Identifier.Name) == 0;
	}



	public override int opCmp(Object o) const
	{
		if (o is null) return -1;
		assert(typeid(this) == typeid(o));
		auto Identifier = cast(CIdentifier)o;
		return icmp(Name, Identifier.Name);
	}
}