﻿module QBasic.Program.CValue;
import std.algorithm;
import QBasic.Interpreter.CReference, QBasic.Program.CName, QBasic.Program.CType;

class CValue
{
	public immutable EPrimitiveType Type;
	private immutable int mIntegralValue;
	private immutable float mSingleValue;
	private immutable ubyte[] mStringValue;
	private CReference mReference;

	public static CValue Integer(int aValue)
	in
	{
		assert(aValue >= -32768 && aValue < 32767);
	}
	body
	{
		return new CValue(EPrimitiveType.Integer, aValue, float.nan, null, null);
	}



	public static CValue Long(int aValue)
	{
		return new CValue(EPrimitiveType.Long, aValue, float.nan, null, null);
	}



	public static CValue Integral(EPrimitiveType aType, int aValue)
	in
	{
		assert((aType == EPrimitiveType.Integer && (aValue >= -32768 && aValue < 32767)) ||
				aType == EPrimitiveType.Long);
	}
	body
	{
		return new CValue(aType, aValue, float.nan, null, null);
	}



	public static CValue Single(float aValue)
	{
		return new CValue(EPrimitiveType.Single, 0, aValue, null, null);
	}



	public static CValue String(immutable(ubyte[]) aValue)
	{
		return new CValue(EPrimitiveType.String, 0, float.nan, aValue, null);
	}



	public static CValue ArrayReference(CReference aReference)
	in
	{
		assert(aReference.Subscripts is null && aReference.RecordOffset == 0);
	}
	body
	{
		return new CValue(EPrimitiveType.ArrayReference, 0, float.nan, null, aReference);
	}



	public static CValue Reference(CReference aReference)
	{
		return new CValue(EPrimitiveType.Reference, 0, float.nan, null, aReference);
	}



	private this(EPrimitiveType aType, int aIntegralValue, float aSingleValue,
			immutable(ubyte[]) aStringValue, CReference aReference)
	{
		Type = aType;
		mIntegralValue = aIntegralValue;
		mSingleValue = aSingleValue;
		mStringValue = aStringValue;
		mReference = aReference;
	}



	public int IntegralValue() const
	in
	{
		assert(Type == EPrimitiveType.Integer || Type == EPrimitiveType.Long);
	}
	body
	{
		return mIntegralValue;
	}



	public float SingleValue() const
	in
	{
		assert(Type == EPrimitiveType.Single);
	}
	body
	{
		return mSingleValue;
	}



	public immutable(ubyte[]) StringValue() const
	in
	{
		assert(Type == EPrimitiveType.String);
	}
	body
	{
		return mStringValue;
	}



	public CReference Reference()
	in
	{
		assert(canFind([ EPrimitiveType.ArrayReference, EPrimitiveType.Reference ], Type));
	}
	body
	{
		return mReference;
	}

	public const(CReference) Reference() const
	in
	{
		assert(canFind([ EPrimitiveType.ArrayReference, EPrimitiveType.Reference ], Type));
	}
	body
	{
		return mReference;
	}
}