﻿module Engine.CControllerWrapper;
import Engine.ILogger;
import Tractor.Controller, Tractor.Tractor;

class CControllerWrapper
{
	private ILogger mLogger;
	private IController mController;

	public this(ILogger aLogger, IController aController)
	in
	{
		assert(aLogger !is null && aController !is null);
	}
	body
	{
		mLogger = aLogger;
		mController = aController;
	}



	public bool IsError()
	{
		return mController is null;
	}



	public bool GetOptions(out CControllerOptions aOptions)
	{
		try
		{
			aOptions = mController.GetOptions();
		}
		catch (Throwable t)
		{
			mLogger.Log("Controller GetOptions() failed\n" ~ t.toString());
			delete mController;
			mController = null;
			return false;
		}
		return true;
	}



	public bool Initialise(ITractor aTractor)
	{
		try
		{
			mController.Initialise(aTractor);
		}
		catch (Throwable t)
		{
			mLogger.Log("Controller Initialise() failed\n" ~ t.toString());
			delete mController;
			mController = null;
			return false;
		}
		return true;
	}



	public bool Uninitialise()
	{
		bool Result = true;
		try
		{
			delete mController;
		}
		catch (Throwable t)
		{
			mLogger.Log("Controller Uninitialise() failed\n" ~ t.toString());
			Result = false;
		}
		mController = null;
		return Result;
	}



	public bool OnFrame()
	{
		try
		{
			mController.OnFrame();
		}
		catch (Throwable t)
		{
			mLogger.Log("Controller OnFrame() failed\n" ~ t.toString());
			delete mController;
			mController = null;
			return false;
		}
		return true;
	}



	public bool OnKeyDown(EKey aKey)
	{
		try
		{
			mController.OnKeyDown(aKey);
		}
		catch (Throwable t)
		{
			mLogger.Log("Controller OnKeyDown() failed\n" ~ t.toString());
			delete mController;
			mController = null;
			return false;
		}
		return true;
	}



	public bool OnKeyUp(EKey aKey)
	{
		try
		{
			mController.OnKeyUp(aKey);
		}
		catch (Throwable t)
		{
			mLogger.Log("Controller OnKeyUp() failed\n" ~ t.toString());
			delete mController;
			mController = null;
			return false;
		}
		return true;
	}
}