﻿module Engine.CFileSystem;
import std.algorithm, std.conv, std.file, std.path, std.string;
import win32.windows, win32.shlobj, win32.shlwapi;
import DWin32.knownfolders;
import Engine.IFileSystem;

class CFileSystem : IFileSystem
{
	private string mLocalDirectoryName;

	public void SetLocalDirectoryName(string aLocalDirectoryName)
	{
		mLocalDirectoryName = aLocalDirectoryName;
	}



	public string GetDataFilePath(string aFileName)
	{
		return buildPath(GetExecutablePath(), "Data", aFileName);
	}



	private string GetExecutablePath()
	{
		DWORD Size = MAX_PATH;
		wchar[] FilePath = new wchar[Size + 1];
		while (true)
		{
			DWORD Result = GetModuleFileName(null, FilePath.ptr, Size + 1);
			assert(Result != 0 && Result <= Size);
			if (Result == Size)
			{
				Size *= 2;
				FilePath = new wchar[Size + 1];
			}
			else
				break;
		}
		bool PRFSResult = 0 != PathRemoveFileSpec(FilePath.ptr);
		assert(PRFSResult);
		sizediff_t Index = indexOf(FilePath, '\0');
		assert(Index != -1);
		return to!string(FilePath[0 .. Index]);
	}



	public string GetLocalFilePath(string aFileName)
	in
	{
		assert(mLocalDirectoryName !is null);
	}
	body
	{
		return buildPath(GetLocalDirectoryPath(), aFileName);
	}



	public string GetLocalDirectoryPath()
	{
		wchar* PathStr;
		SHGetKnownFolderPath(&FOLDERID_RoamingAppData, 0, null, &PathStr);
		int Index;
		for (int i = 0; ; i++)
		{
			if (PathStr[i] == '\0')
			{
				Index = i;
				break;
			}
		}
		string Path = PathStr[0 .. Index].to!string;
		CoTaskMemFree(PathStr);
		if (!exists(Path))
			throw new Exception("Local data base directory does not exist");
		string LocalDirectoryPath = buildPath(Path, mLocalDirectoryName);
		if (!exists(LocalDirectoryPath))
			mkdir(LocalDirectoryPath);
		else if (!isDir(LocalDirectoryPath))
			throw new Exception("Local data directory already exists as a file");
		return LocalDirectoryPath;
	}
}