﻿module Engine.CInputModule;
import win32.windows;
import Core.EKey;
import Engine.CEngine, Engine.CMainWindow;

class CInputModule : IInputHandler
{
	private CEngine mEngine;
	private bool[256] mKeyPressed;

	public this(CEngine aEngine)
	{
		mEngine = aEngine;
	}



	public void OnKeyDown(uint aVirtualKey)
	{
		EKey Key;
		if (GetKey(aVirtualKey, Key))
		{
			if (!mEngine.OnKeyDown(Key))
				return;
		}
		if (aVirtualKey <= 255)
			mKeyPressed[aVirtualKey] = true;
	}



	public void OnKeyUp(uint aVirtualKey)
	{
		EKey Key;
		if (GetKey(aVirtualKey, Key))
		{
			if (!mEngine.OnKeyUp(Key))
				return;
		}
	}



	public bool GetKey(uint aVirtualKey, out EKey aKey)
	{
		switch (aVirtualKey)
		{
			case VK_LEFT:
				aKey = EKey.Left;
				return true;
			case VK_RIGHT:
				aKey = EKey.Right;
				return true;
			case VK_UP:
				aKey = EKey.Up;
				return true;
			case VK_DOWN:
				aKey = EKey.Down;
				return true;
			case VK_RETURN:
				aKey = EKey.Enter;
				return true;
			case VK_SPACE:
				aKey = EKey.Space;
				return true;
			case VK_ESCAPE:
				aKey = EKey.Escape;
				return true;
			case VK_BACK:
				aKey = EKey.Backspace;
				return true;
			case VK_TAB:
				aKey = EKey.Tab;
				return true;
			case VK_SHIFT:
			case VK_LSHIFT:
				aKey = EKey.LeftShift;
				return true;
			case VK_RSHIFT:
				aKey = EKey.RightShift;
				return true;
			case VK_CONTROL:
			case VK_LCONTROL:
				aKey = EKey.LeftControl;
				return true;
			case VK_RCONTROL:
				aKey = EKey.RightControl;
				return true;
			case VK_MENU:
			case VK_LMENU:
				aKey = EKey.LeftAlt;
				return true;
			case VK_RMENU:
				aKey = EKey.RightAlt;
				return true;
			case cast(uint)'A':
				aKey = EKey.A;
				return true;
			case cast(uint)'B':
				aKey = EKey.B;
				return true;
			case cast(uint)'C':
				aKey = EKey.C;
				return true;
			case cast(uint)'D':
				aKey = EKey.D;
				return true;
			case cast(uint)'E':
				aKey = EKey.E;
				return true;
			case cast(uint)'F':
				aKey = EKey.F;
				return true;
			case cast(uint)'G':
				aKey = EKey.G;
				return true;
			case cast(uint)'H':
				aKey = EKey.H;
				return true;
			case cast(uint)'I':
				aKey = EKey.I;
				return true;
			case cast(uint)'J':
				aKey = EKey.J;
				return true;
			case cast(uint)'K':
				aKey = EKey.K;
				return true;
			case cast(uint)'L':
				aKey = EKey.L;
				return true;
			case cast(uint)'M':
				aKey = EKey.M;
				return true;
			case cast(uint)'N':
				aKey = EKey.N;
				return true;
			case cast(uint)'O':
				aKey = EKey.O;
				return true;
			case cast(uint)'P':
				aKey = EKey.P;
				return true;
			case cast(uint)'Q':
				aKey = EKey.Q;
				return true;
			case cast(uint)'R':
				aKey = EKey.R;
				return true;
			case cast(uint)'S':
				aKey = EKey.S;
				return true;
			case cast(uint)'T':
				aKey = EKey.T;
				return true;
			case cast(uint)'U':
				aKey = EKey.U;
				return true;
			case cast(uint)'V':
				aKey = EKey.V;
				return true;
			case cast(uint)'W':
				aKey = EKey.W;
				return true;
			case cast(uint)'X':
				aKey = EKey.X;
				return true;
			case cast(uint)'Y':
				aKey = EKey.Y;
				return true;
			case cast(uint)'Z':
				aKey = EKey.Z;
				return true;
			case cast(uint)'0':
				aKey = EKey.k0;
				return true;
			case cast(uint)'1':
				aKey = EKey.k1;
				return true;
			case cast(uint)'2':
				aKey = EKey.k2;
				return true;
			case cast(uint)'3':
				aKey = EKey.k3;
				return true;
			case cast(uint)'4':
				aKey = EKey.k4;
				return true;
			case cast(uint)'5':
				aKey = EKey.k5;
				return true;
			case cast(uint)'6':
				aKey = EKey.k6;
				return true;
			case cast(uint)'7':
				aKey = EKey.k7;
				return true;
			case cast(uint)'8':
				aKey = EKey.k8;
				return true;
			case cast(uint)'9':
				aKey = EKey.k9;
				return true;
			case VK_INSERT:
				aKey = EKey.Insert;
				return true;
			case VK_DELETE:
				aKey = EKey.Delete;
				return true;
			case VK_HOME:
				aKey = EKey.Home;
				return true;
			case VK_END:
				aKey = EKey.End;
				return true;
			case VK_PRIOR:
				aKey = EKey.PageUp;
				return true;
			case VK_NEXT:
				aKey = EKey.PageDown;
				return true;
			default:
				return false;
		}
	}



	public uint GetVirtualKey(EKey aKey)
	{
		final switch (aKey)
		{
			case EKey.Left:
				return VK_LEFT;
			case EKey.Right:
				return VK_RIGHT;
			case EKey.Up:
				return VK_UP;
			case EKey.Down:
				return VK_DOWN;
			case EKey.Enter:
				return VK_RETURN;
			case EKey.Space:
				return VK_SPACE;
			case EKey.Escape:
				return VK_ESCAPE;
			case EKey.Backspace:
				return VK_BACK;
			case EKey.Tab:
				return VK_TAB;
			case EKey.LeftShift:
				return VK_LSHIFT;
			case EKey.RightShift:
				return VK_RSHIFT;
			case EKey.LeftControl:
				return VK_LCONTROL;
			case EKey.RightControl:
				return VK_RCONTROL;
			case EKey.LeftAlt:
				return VK_LMENU;
			case EKey.RightAlt:
				return VK_RMENU;
			case EKey.A:
				return cast(uint)'A';
			case EKey.B:
				return cast(uint)'B';
			case EKey.C:
				return cast(uint)'C';
			case EKey.D:
				return cast(uint)'D';
			case EKey.E:
				return cast(uint)'E';
			case EKey.F:
				return cast(uint)'F';
			case EKey.G:
				return cast(uint)'G';
			case EKey.H:
				return cast(uint)'H';
			case EKey.I:
				return cast(uint)'I';
			case EKey.J:
				return cast(uint)'J';
			case EKey.K:
				return cast(uint)'K';
			case EKey.L:
				return cast(uint)'L';
			case EKey.M:
				return cast(uint)'M';
			case EKey.N:
				return cast(uint)'N';
			case EKey.O:
				return cast(uint)'O';
			case EKey.P:
				return cast(uint)'P';
			case EKey.Q:
				return cast(uint)'Q';
			case EKey.R:
				return cast(uint)'R';
			case EKey.S:
				return cast(uint)'S';
			case EKey.T:
				return cast(uint)'T';
			case EKey.U:
				return cast(uint)'U';
			case EKey.V:
				return cast(uint)'V';
			case EKey.W:
				return cast(uint)'W';
			case EKey.X:
				return cast(uint)'X';
			case EKey.Y:
				return cast(uint)'Y';
			case EKey.Z:
				return cast(uint)'Z';
			case EKey.k0:
				return cast(uint)'0';
			case EKey.k1:
				return cast(uint)'1';
			case EKey.k2:
				return cast(uint)'2';
			case EKey.k3:
				return cast(uint)'3';
			case EKey.k4:
				return cast(uint)'4';
			case EKey.k5:
				return cast(uint)'5';
			case EKey.k6:
				return cast(uint)'6';
			case EKey.k7:
				return cast(uint)'7';
			case EKey.k8:
				return cast(uint)'8';
			case EKey.k9:
				return cast(uint)'9';
			case EKey.Insert:
				return VK_INSERT;
			case EKey.Delete:
				return VK_DELETE;
			case EKey.Home:
				return VK_HOME;
			case EKey.End:
				return VK_END;
			case EKey.PageUp:
				return VK_PRIOR;
			case EKey.PageDown:
				return VK_NEXT;
		}
	}
}