﻿module Fonts.CFontModule;
import std.utf;
import freetype.ft;
import Tractor.Fonts, Engine.IFileSystem;

class CFontModule : IFontModule
{
	private IFileSystem mFileSystem;
	private bool mDerelictFTInitialised = false;
	private FT_Library mFreeTypeLibrary;
	private FT_Face[] mFaces;

	public this(IFileSystem aFileSystem)
	{
		mFileSystem = aFileSystem;
	}



	public ~this()
	{
		foreach (FT_Face Face; mFaces)
		{
			FT_Error FTResult = FT_Done_Face(Face);
			assert(FTResult == 0);
		}
	}



	public IFont LoadFont(string aFileName)
	{
		if (!mDerelictFTInitialised)
		{
			mDerelictFTInitialised = true;
			FT_Error FIFResult = FT_Init_FreeType(&mFreeTypeLibrary);
			if (FIFResult != 0)
				throw new Exception("Failed to initialise FreeType library.");
		}
		string Path = mFileSystem.GetDataFilePath(aFileName);
		FT_Face Face;
		FT_Error Error = FT_New_Face(mFreeTypeLibrary, toUTFz!(immutable char*)(Path), 0,
				&Face);
		if (Error != 0)
			throw new Exception("Failed to load font: " ~ aFileName);
		mFaces ~= Face;
		return new CFont(Face);
	}
}



class CFont : IFont
{
	public FT_Face mFace;

	public this(FT_Face aFace)
	{
		mFace = aFace;
	}



	public FT_Face Face()
	{
		return mFace;
	}



	public FT_F26Dot6 YSize(FT_F26Dot6 aSize)
	{
		FT_Error FTResult = FT_Set_Char_Size(mFace, 0, aSize, 0, 0);
		assert(FTResult == 0);
		return (mFace.ascender - mFace.descender) * mFace.size.metrics.y_scale / 65536;
	}



	public FT_F26Dot6 AscenderSize(FT_F26Dot6 aSize)
	{
		FT_Error FTResult = FT_Set_Char_Size(mFace, 0, aSize, 0, 0);
		assert(FTResult == 0);
		return mFace.ascender * mFace.size.metrics.y_scale / 65536;
	}



	public FT_F26Dot6 DescenderSize(FT_F26Dot6 aSize)
	{
		FT_Error FTResult = FT_Set_Char_Size(mFace, 0, aSize, 0, 0);
		assert(FTResult == 0);
		return mFace.descender * mFace.size.metrics.y_scale / 65536;
	}



	public FT_F26Dot6 GetCharacterXSize(dchar aChar, FT_Vector aPosition, FT_F26Dot6 aSize)
	{
		FT_Error FTResult = FT_Set_Char_Size(mFace, 0, aSize, 0, 0);
		assert(FTResult == 0);
		FT_Set_Transform(mFace, null, &aPosition);
		FTResult = FT_Load_Char(mFace, aChar, FT_LOAD_RENDER);
		assert(FTResult == 0);
		return mFace.glyph.advance.x;
	}
}