﻿module Shaders.Shaders;
import std.utf;
import derelict.opengl3.gl3;
import Engine.CTractorException, Engine.ILogger;
import Tractor.Shaders;

class CShader : IShader
{
	private GLuint mID;

	public this(GLuint aID)
	{
		mID = aID;
	}



	public ~this()
	{
		glDeleteShader(mID);
	}



	public uint ID()
	{
		return mID;
	}
}



class CBuiltinVertexShader : CShader, IVertexShader
{
	public this(GLuint aID)
	{
		super(aID);
	}
}



class CBuiltinFragmentShader : CShader, IFragmentShader
{
	public this(GLuint aID)
	{
		super(aID);
	}
}



class CCustomVertexShader : CShader, IVertexShader
{
	public this(GLuint aID)
	{
		super(aID);
	}
}



class CCustomFragmentShader : CShader, IFragmentShader
{
	public this(GLuint aID)
	{
		super(aID);
	}
}



class CShaderProgram : IShaderProgram
{
	private ILogger mLogger;
	private GLuint mID;

	public this(ILogger aLogger, GLuint aID)
	{
		mLogger = aLogger;
		mID = aID;
	}



	public ~this()
	{
		glDeleteProgram(mID);
	}



	public uint ID()
	{
		return mID;
	}



	public void BindAttribute(int aIndex, string aName)
	{
		glBindAttribLocation(mID, aIndex, toUTFz!(GLchar*)(aName));
	}



	public void Link()
	{
		glLinkProgram(mID);
		GLint LinkStatus;
		glGetProgramiv(mID, GL_LINK_STATUS, &LinkStatus);
		if (LinkStatus == GL_FALSE)
		{
			mLogger.Log("Failed to link program");
			char[] InfoLog = new char[1000];
			GLsizei Length;
			glGetProgramInfoLog(mID, InfoLog.length, &Length, InfoLog.ptr);
			mLogger.Log(cast(string)InfoLog[0 .. Length]);
			throw new CTractorException();
		}
	}
}