﻿module Textures.CTextureModule;
import std.exception, std.utf;
import derelict.devil.il, derelict.opengl3.gl3;
import Engine.IFileSystem;
import Tractor.Textures;

class CTextureModule : ITextureModule
{
	private IFileSystem mFileSystem;
	private bool mDerelictILInitialised = false;
	private GLuint[] mTextureNames;

	public this(IFileSystem aFileSystem)
	{
		mFileSystem = aFileSystem;
	}



	public ~this()
	{
		glBindTexture(GL_TEXTURE_2D, 0);
		if (mTextureNames.length != 0)
			glDeleteTextures(mTextureNames.length, mTextureNames.ptr);
	}



	public ITexture LoadImage(string aFileName)
	{
		if (!mDerelictILInitialised)
		{
			mDerelictILInitialised = true;
			DerelictIL.load();
			ilInit();
		}
		string Path = mFileSystem.GetDataFilePath(aFileName);
		ILuint ImageHandle;
		ilLoadImage(toUTFz!(char*)(Path));
		enforce(ilGetError() == IL_NO_ERROR);
		int XSize = ilGetInteger(IL_IMAGE_WIDTH);
		int YSize = ilGetInteger(IL_IMAGE_HEIGHT);
		int DataXSize = XSize;
		if (DataXSize % 4 != 0)
			DataXSize += 4 - DataXSize % 4;
		ubyte[] ImageData = new ubyte[DataXSize * YSize * 4];
		ilCopyPixels(0, 0, 0, DataXSize, YSize, 1, IL_RGBA, IL_UNSIGNED_BYTE, ImageData.ptr);
		enforce(ilGetError() == IL_NO_ERROR);
		GLuint TextureName;
		glGenTextures(1, &TextureName);
		mTextureNames ~= TextureName;
		glBindTexture(GL_TEXTURE_2D, TextureName);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, DataXSize, YSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, ImageData.ptr);
		assert(glGetError() == GL_NO_ERROR);
		delete ImageData;
		return new CTexture(TextureName, XSize, cast(float)XSize / DataXSize, YSize);
	}



	public ITexture CreateTexture(const(uint[]) aData, int aXSize, int aYSize)
	{
		GLuint TextureName;
		glGenTextures(1, &TextureName);
		mTextureNames ~= TextureName;
		glBindTexture(GL_TEXTURE_2D, TextureName);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, aXSize, aYSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, aData.ptr);
		assert(glGetError() == GL_NO_ERROR);
		return new CTexture(TextureName, aXSize, 1, aYSize);
	}
}



class CTexture : ITexture
{
	private GLuint mID;
	private int mXSize;
	private float mTextureXSize;
	private int mYSize;

	public this(GLuint aID, int aXSize, float aTextureXSize, int aYSize)
	{
		mID = aID;
		mXSize = aXSize;
		mTextureXSize = aTextureXSize;
		mYSize = aYSize;
	}



	public uint ID()
	{
		return mID;
	}



	public int XSize()
	{
		return mXSize;
	}



	public int YSize()
	{
		return mYSize;
	}



	public float TextureXSize()
	{
		return mTextureXSize;
	}



	public float TextureYSize()
	{
		return 1;
	}



	public void SetData(const(uint[]) aData)
	{
		glBindTexture(GL_TEXTURE_2D, mID);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, mXSize, mYSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, aData.ptr);
	}
}