﻿module glwh.glwh;
import derelict.opengl3.gl, derelict.opengl3.gl3, derelict.opengl3.wgl, derelict.util.wintypes;

// Accepted as an attribute name in <*attribList>:
enum int WGL_CONTEXT_MAJOR_VERSION_ARB = 0x2091;
enum int WGL_CONTEXT_MINOR_VERSION_ARB = 0x2092;
enum int WGL_CONTEXT_LAYER_PLANE_ARB = 0x2093;
enum int WGL_CONTEXT_FLAGS_ARB = 0x2094;
enum int WGL_CONTEXT_PROFILE_MASK_ARB = 0x9126;

// Accepted as bits in the attribute value for WGL_CONTEXT_FLAGS in <*attribList>:
enum int WGL_CONTEXT_DEBUG_BIT_ARB = 0x0001;
enum int WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB = 0x0002;

// Accepted as bits in the attribute value for WGL_CONTEXT_PROFILE_MASK_ARB in <*attribList>:
enum int WGL_CONTEXT_CORE_PROFILE_BIT_ARB = 0x00000001;
enum int WGL_CONTEXT_COMPATIBILITY_PROFILE_BIT_ARB = 0x00000002;

// New errors returned by GetLastError:
enum int ERROR_INVALID_VERSION_ARB = 0x2095;
enum int ERROR_INVALID_PROFILE_ARB = 0x2096;



extern (Windows)
{
	alias HGLRC function(HDC hDC, HGLRC hShareContext, const(int)* attribList) FwglCreateContextAttribsARB;
}



/**
Creates an OpenGL rendering context for a device context, sets it as the current rendering
context for the device context (by using wglMakeCurrent()) and initialises DerelictGL3.
The pixel format for the device context will be set to 24-bit RGBA pixels with the specified
number of bits for the depth buffer.
@return The rendering context if it was successfully created, or NULL if it could not be
created.
*/
public HGLRC glwhCreateRenderingContextForDevice(GLVersion aVersion, HDC aDC,
		ubyte aDepthBits = 0)
{
	DerelictGL3.load();
	PIXELFORMATDESCRIPTOR pfd;
	pfd.nSize = PIXELFORMATDESCRIPTOR.sizeof;
	pfd.nVersion = 1;
	pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
	pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.cColorBits = 24;
	pfd.cDepthBits = aDepthBits;
	int PixelFormat = ChoosePixelFormat(aDC, &pfd);
	if (PixelFormat == 0)
		return null;
	int DPFResult = DescribePixelFormat(aDC, PixelFormat, PIXELFORMATDESCRIPTOR.sizeof, &pfd);
	assert(DPFResult != 0);
	if (!(pfd.iPixelType == PFD_TYPE_RGBA && pfd.cColorBits >= 24 && (aDepthBits == 0 ||
			pfd.cDepthBits >= aDepthBits)))
		return null;
	bool SPFResult = 0 != SetPixelFormat(aDC, PixelFormat, &pfd);
	if (!SPFResult)
		return null;
	HGLRC TempRenderingContext = wglCreateContext(aDC);
	if (TempRenderingContext == null)
		return null;
	bool WMCResult = 0 != wglMakeCurrent(aDC, TempRenderingContext);
	assert(WMCResult);
	assert(glGetError() == GL_NO_ERROR);
	if (aVersion < GLVersion.GL30)
		return TempRenderingContext;
	int[] Attribs = [
		WGL_CONTEXT_MAJOR_VERSION_ARB, cast(int)aVersion / 10,
		WGL_CONTEXT_MINOR_VERSION_ARB, cast(int)aVersion % 10,
		WGL_CONTEXT_FLAGS_ARB, WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB,
		0
	];
	FwglCreateContextAttribsARB wglCreateContextAttribsARB = null;
	wglCreateContextAttribsARB = cast(FwglCreateContextAttribsARB)wglGetProcAddress("wglCreateContextAttribsARB");
	bool WDCResult;
	if (wglCreateContextAttribsARB == null)
	{
		WMCResult = 0 != wglMakeCurrent(null, null);
		assert(WMCResult);
		WDCResult = 0 != wglDeleteContext(TempRenderingContext);
		assert(WDCResult);
		return null;
	}
	HGLRC RenderingContext = wglCreateContextAttribsARB(aDC, null, Attribs.ptr);
	WMCResult = 0 != wglMakeCurrent(null, null);
	assert(WMCResult);
	WDCResult = 0 != wglDeleteContext(TempRenderingContext);
	assert(WDCResult);
	if (RenderingContext == null)
		return null;
	if (0 == wglMakeCurrent(aDC, RenderingContext))
	{
		WDCResult = 0 != wglDeleteContext(RenderingContext);
		assert(WDCResult);
		return null;
	}
	if (DerelictGL3.reload() < aVersion)
		return null;
	return RenderingContext;
}



public void glwhDeleteRenderingContext(HGLRC aRenderingContext)
{
	bool WMCResult = 0 != wglMakeCurrent(null, null);
	assert(WMCResult);
	bool WDCResult = 0 != wglDeleteContext(aRenderingContext);
	assert(WDCResult);
}